#!/bin/bash

ffoo import core

FFOO_INI_SUFFIX=".ini"


__iniread__cat() {
    # cat without starting a process
    while IFS= read line;
    do echo "$line"; done
}

__iniread__flt_comments() {
    grep -v -e "^[[:space:]]*[#;]" -e "^[[:space:]]*$"
}

__iniread__flt_key() {
    # filter values from key=value pair
    local line
    local key
    local value
    while IFS= read line;
    do
        line=$(sed -re 's/^\s*//' <<<"$line")
        key=$(sed -re 's/\s*=.*//; s/^\s*//' <<<"$line")
        if $strict;
        then
            value=$(sed -re 's/[^=]*=//' <<<"$line")
        else
            value=$(sed -re 's/[^=]*=\s*//' <<<"$line")
        fi
        if test "$key" = "$wantkey";
        then
            echo "$value"
        fi
    done
}

__iniread__flt_section() {
    # filter per section
    local section_ok=false
    local line
    while IFS= read line;
    do
        if grep -qse "^\[$wantsection\]" <<<"$line";
        then
            section_ok=true
        elif grep -qse "^\[" <<<"$line";
        then
            section_ok=false
        elif $section_ok;
        then
            $strict || line="$(sed -re 's/^\s*//' <<<"$line")"
            echo "$line"
        fi
    done
}

__iniread__merge() {
    if test -z "$1";
    then                # guess filename from section head
        local guess="$(cut -d. -f1 <<<"$wantsection")$FFOO_INI_SUFFIX"
        debug -v guess
        __iniread__merge "$guess"
    fi
    local arg trydir trypath
    for arg in "$@";
    do
        debug -v arg
        case $arg in
            -|*/*)      # stdin, or path (with slash)
                cat $arg
            ;;
        *)              # name given, find all its incarnations
            debug -v FFOO_INI_PATH
            echo "$FFOO_INI_PATH" \
              | tr ':' '\n' \
              | while read trydir;
                do
                    trypath="$trydir/$arg"
                    debug -v trypath
                    cat $trypath 2>/dev/null
                done
            ;;
        esac
    done
    true
}

iniread() {
    #
    # A flexible (not just) .ini file reader
    #
    # Can read anything from commented plaintext to keyless section INI,
    # sectionless keyed INI or sectioned keyed INI, including multi-line
    # and whitespace-safe chunks.
    #
    # Is constantly becoming smarter and smarter.
    #
    local wantsection=""
    local wantkey=""
    local strict=false
    local section_ok=true
    local one_line=false
    while true;
    do
        case $1 in
            --)
                break
                ;;
            -1|--one-line)
                one_line=true
                shift 1
                ;;
            -k|--key)
                wantkey="$2"
                shift 2
                ;;
            -p|--path)
                wantkey="$(echo $2 | rev | cut -d. -f1 | rev)"
                wantsection="${2%.$wantkey}"
                shift 2
                ;;
            -s|--section)
                wantsection="$2"
                section_ok=false
                shift 2
                ;;
            -S|--strict)
                strict=true
                shift 1
                ;;
            -l|--list-sections)
                #TODO: list sections
                warn "--list-sections is not implemented"
                shift
                ;;
            -L|--list-keys)
                #TODO: list keys as section/key (with -s, list keys only)
                warn "--list-keys is not implemented"
                shift
                ;;
            "")
                break
                ;;
            *)
                break
                ;;
        esac
    done

    flt_section=__iniread__cat
    flt_key=__iniread__cat
    limit_line=__iniread__cat
    test -n "$wantsection" && flt_section=__iniread__flt_section
    test -n "$wantkey" && flt_key=__iniread__flt_key
    $one_line && limit_line="head -1"

    debug -v wantkey wantsection
    debug "\$@='$@'"

    __iniread__merge "$@" \
        | __iniread__flt_comments \
        | $flt_section \
        | $flt_key \
        | $limit_line \
        | grep .
}
